#!/usr/bin/perl -w
#Filename:
#Author: Li Cuiping
#Date: 2022-06-03
#Modified:
#Description: ﶻ߱ע.
my $version=1.00;

use strict;
use Getopt::Long;
use File::Basename;

my %opts;
GetOptions(\%opts,"i=s","refdir=s","dir=s","h");
if (!(defined $opts{i} and defined $opts{refdir} and defined $opts{dir}) || defined $opts{h}) { #necessary arguments
&usage;
}


my $vcf_tmp=$opts{dir}."/tmp.vcf";
open VCFT,">$vcf_tmp";
print VCFT "##fileformat=VCFv4.2\n";
print VCFT "##ANN: Annotation detail by VEP\n";
open IN,"<$opts{i}";
while (my $aline=<IN>) {
	next if($aline=~/^\#\#/);
	chomp $aline;
	my @arr=split /\t/,$aline;
	if ($aline=~/^\#/) {
		$arr[7]="Annotation";
	}else{
		$arr[2]=$arr[1].":".$arr[3].">".$arr[4];
	}
	$aline=join "\t",@arr;
	print VCFT "$aline\n";
}
close IN;
close VCFT;

my $vep_out=$opts{dir}."/out.vep";
`export PERL5LIB=\$PERL5LIB`;
`vep -offline -i $vcf_tmp -o $vep_out --species Monkeypox --cache_version 1 --dir $opts{refdir} --distance 0 --force`;

my %vep_anno;
open VEP,"<$vep_out";
while (my $aline=<VEP>) {
	next if($aline=~/^\#/);
	chomp $aline;
	my @arr=split /\t/,$aline;
	$arr[6]=~s/3_prime_UTR_variant//g;
	$arr[6]=~s/5_prime_UTR_variant//g;
	$arr[6]=~s/,,/,/g;
	if ($arr[6]=~/^,/) {
		$arr[6]=~s/,//;
	}
	if ($arr[6]=~/,$/) {
		chop $arr[6];
	}

	if ($arr[6]=~/intergenic_variant/) {
		$vep_anno{$arr[0]}.=";ANN=$arr[6]";
	}elsif ($arr[6]=~/stream/) {
		$arr[13]=~/DISTANCE=(\d+);/;
		my $dist=$1;
		$vep_anno{$arr[0]}.=";ANN=$arr[6],DISTANCE=$dist,$arr[3]";
	}else{
		my ($cc,$pp);
		if ($arr[10] eq "-") {
			$vep_anno{$arr[0]}.=";ANN=$arr[6]";
		}else{
			my @brr=split /:/,$arr[1];
			$arr[11]=~s/\//$brr[1]/;
			$cc=$arr[3].":c.".$arr[11];
			if ($arr[10]!~/\//) {
				$pp=$arr[3].":p.".$arr[10].$arr[9];
			}else{
				$arr[10]=~s/\//$arr[9]/;
				$pp=$arr[3].":p.".$arr[10];
			}
			$vep_anno{$arr[0]}.=";ANN=$arr[6],$pp,$cc";
		}
	}
}
close VEP;

my $filename=basename($opts{i});
my $vcf_out=$opts{dir}."/".$filename.".annotated";
open VCF,">$vcf_out";
open VIN,"<$vcf_tmp";
while (my $aline=<VIN>) {
	chomp $aline;
	if ($aline!~/^\#/) {
		my @arr=split /\t/,$aline;
		$vep_anno{$arr[2]}=~s/;//;
		$arr[7]=$vep_anno{$arr[2]};
		$aline=join "\t",@arr;
	}
	print VCF "$aline\n";
}
close VIN;
close VCF;

`rm -rf $vep_out "$vep_out\_summary.html" $vcf_tmp`;




sub usage{
print <<"USAGE";
Description: ﶻ߱ע.
Usage:
$0 -refdir -i -dir
options:
-refdir   input reference directory
-i input  virus variants vcf file
-dir      output directory file
-h        help
USAGE
exit(1);
}
